<?php
/**
 * @Author: José Elías Gutierrez
 * @Date:   2017-04-10 10:58:40
 * @Last Modified by:   José Elías Gutierrez
 * @Last Modified time: 2017-06-19 16:40:33
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

function jebe_add_variation_price_fields($loop, $variation){
	$_booking_rate = get_post_meta($variation->ID, '_booking_rate', true);
    $rates = get_option('jebe_rates');

    $_booking_extra_group = get_post_meta($variation->ID, '_booking_extra_group', true);
    $extra_groups = get_option('jebe_extra_groups');
	require(dirname(__FILE__).'/views/variation_extra_fields.php');
}


function jebe_booking_price_fields_save($variation_id, $i){

	// rate
	$booking_rate = $_POST['_var_booking_rate'][$i];
	if( !empty( $booking_rate ) )
		update_post_meta( $variation_id, '_booking_rate', esc_attr( $booking_rate ) );

    // rate
    $booking_extra_group = $_POST['_var_booking_extra_group'][$i];
    if( !empty( $booking_extra_group ) )
        update_post_meta( $variation_id, '_booking_extra_group', esc_attr( $booking_extra_group ) );

}



function jebe_order_display_product_dates($item_id, $item, $_product ) {
    global $wpdb;

    if ( ! isset( $item['product_id'] ) ) {
        return;
    }
    
    $product_id   = $item['product_id'];
    $variation_id = $item['variation_id'];

    $start_hour_set = wc_get_order_item_meta( $item_id, '_ebs_start_hour_display' );
    $end_hour_set   = wc_get_order_item_meta( $item_id, '_ebs_end_hour_display' );

    $start_hour_text = __( 'Start', 'jebe_easy_booking' );
    $end_hour_text   = __( 'End', 'jebe_easy_booking' );

    $item_order_meta_table = $wpdb->prefix . 'woocommerce_order_itemmeta';
    
    if ( ! empty( $start_hour_set ) ) {

        // Get meta ids from the database
        $start_display_meta_id = $wpdb->get_var( $wpdb->prepare(
            "SELECT `meta_id` FROM $item_order_meta_table WHERE `order_item_id` = %d AND `meta_key` LIKE %s",
            $item_id, '_ebs_start_hour_display'
        ));

        if ( ! empty( $end_hour_set ) ) {

            $end_display_meta_id = $wpdb->get_var( $wpdb->prepare(
                "SELECT `meta_id` FROM $item_order_meta_table WHERE `order_item_id` = %d AND `meta_key` LIKE %s",
                $item_id, '_ebs_end_hour_display'
            ));

        }

        // Product or variation ID
        $id      = empty( $variation_id ) ? $product_id : $variation_id;
        $product = wc_get_product( $id );

        $start_hour = $start_hour_set;
        $end_hour = $end_hour_set;

        include( 'views/order-item-meta.php' );

    }
}


class JEBE_Settings {
    private $rate_fields = [
        [   'name' => 'hour_2', 
            'title' => 'hour_2', 
        ],
        [   'name' => 'hour_4', 
            'title' => 'hour_4', 
        ],
        [   'name' => 'hour_8', 
            'title' => 'hour_8', 
        ],
        [   'name' => 'day_1', 
            'title' => 'day_1', 
        ],
        [   'name' => 'day_2', 
            'title' => 'day_2', 
        ],
        [   'name' => 'hourextra', 
            'title' => 'hourextra', 
        ],
        [   'name' => 'weekend', 
            'title' => 'weekend', 
        ],
        [   'name' => 'days_3_4', 
            'title' => 'days_3_4', 
        ],
        [   'name' => 'days_5_9', 
            'title' => 'days_5_9', 
        ],
        [   'name' => 'days_10_16', 
            'title' => 'days_10_16', 
        ],
        [   'name' => 'days_17_25', 
            'title' => 'days_17_25', 
        ],
        [   'name' => 'days_26_36', 
            'title' => 'days_26_36', 
        ],
        [   'name' => 'days_37_49', 
            'title' => 'days_37_49', 
        ],
        [   'name' => 'days_50_64', 
            'title' => 'days_50_64', 
        ],
        [   'name' => 'days_65_81', 
            'title' => 'days_65_81', 
        ],
        [   'name' => 'days_82', 
            'title' => 'days_82', 
        ],
    ];

    private $extra_groups_fields = [
        [   'name' => 'extra_group_name', 
            'title' => 'extra_group_name', 
        ]
    ];

	public function __construct() {

		// get plugin options values
		$this->options = get_option('jebe_settings');
		$this->holidays = get_option('jebe_holidays');
        $this->rates = get_option('jebe_rates');
        $this->extra_groups = get_option('jebe_extra_groups');

		// initialize options the first time
		if ( ! $this->options ) {
		
		    $this->options = array(
		    	'jebe_allow_book_in_holiday'	=> 'off',
		    	'jebe_min_bookable_hour'		=> '0',
		    	'jebe_max_bookable_hour'		=> '0',
		    	'jebe_stock_attribute'			=> '',
		    );

		    $this->holidays = array();
            $this->rates = array();
            $this->extra_groups = array();

		    add_option( 'jebe_settings', $this->options );
		    add_option( 'jebe_holidays', $this->holidays );
            add_option( 'jebe_rates', $this->rates );
            add_option( 'jebe_extra_groups', $this->extra_groups );

		}

		if ( is_admin() ) {

			add_action( 'admin_menu', array( $this, 'jebe_add_admin_menu' ), 10 );

			add_action( 'admin_init', array( $this, 'jebe_settings_init' ) );
		}
	}

	public function jebe_settings_init( ) {

		add_action( 'jebe_settings_general_tab', array( $this, 'jebe_general_settings' ), 10);
		add_action( 'jebe_settings_holidays_tab', array( $this, 'jebe_holidays_settings' ), 10);
        add_action( 'jebe_settings_rates_tab', array( $this, 'jebe_rates_settings' ), 10);
        add_action( 'jebe_settings_extra_groups_tab', array( $this, 'jebe_extra_groups_settings' ), 10);

		add_action( 'jebe_settings_show_holidays', array( $this, 'jebe_holidays_dates_display' ), 10);
        add_action( 'jebe_settings_show_rates', array( $this, 'jebe_rates_display' ), 10);
        add_action( 'jebe_settings_show_extra_groups', array( $this, 'jebe_extra_groups_display' ), 10);

		register_setting(
			'jebe_general_settings',
			'jebe_settings', 
			array( $this, 'sanitize_values' )
		);

		add_settings_section(
			'jebe_general_main_settings',
			__( 'General settings', 'j_easy_booking_extended' ),
			array( $this, 'jebe_section_general' ),
			'jebe_general_settings'
		);

		add_settings_field(
			'jebe_global_allow_book_in_holiday',
			__( 'Allow bookings in holidays', 'j_easy_booking_extended' ),
			array( $this, 'jebe_field_allow_book_in_holiday' ),
			'jebe_general_settings',
			'jebe_general_main_settings'
		);

		add_settings_field(
			'jebe_min_bookable_hour',
			__( 'Min bookable hour', 'j_easy_booking_extended' ),
			array( $this, 'jebe_field_min_bookable_hour' ),
			'jebe_general_settings',
			'jebe_general_main_settings'
		);

		add_settings_field(
			'jebe_max_bookable_hour',
			__( 'Max bookable hour', 'j_easy_booking_extended' ),
			array( $this, 'jebe_field_max_bookable_hour' ),
			'jebe_general_settings',
			'jebe_general_main_settings'
		);

		add_settings_field(
			'jebe_stock_attribute',
			__( 'Attribute for stock management', 'j_easy_booking_extended' ),
			array( $this, 'jebe_field_stock_management' ),
			'jebe_general_settings',
			'jebe_general_main_settings'
		);


		// Holidays settings field

		register_setting(
			'jebe_holidays_settings',
			'jebe_holidays', 
			array( $this, 'sanitize_collection' )
		);

		add_settings_section(
			'jebe_holidays_main_settings',
			__( 'Holidays', 'j_easy_booking_extended' ),
			array( $this, 'jebe_section_holiday' ),
			'jebe_holidays_settings'
		);

		add_settings_field(
			'jebe_holiday',
			__( 'Add new holiday', 'j_easy_booking_extended' ),
			array( $this, 'jebe_field_holiday' ),
			'jebe_holidays_settings',
			'jebe_holidays_main_settings'
		);

        register_setting(
            'jebe_rates_settings',
            'jebe_rates', 
            array( $this, 'sanitize_collection' )
        );

        add_settings_section(
            'jebe_rates_main_settings',
            __( 'Rates', 'j_easy_booking_extended' ),
            array( $this, 'jebe_section_rate' ),
            'jebe_rates_settings'
        );

        # rate fields

        add_settings_field(
            'jebe_rate_name',
            __( 'Add new rate', 'j_easy_booking_extended' ),
            array( $this, 'jebe_field_rate_name' ),
            'jebe_rates_settings',
            'jebe_rates_main_settings'
        );

        foreach ($this->rate_fields as $key => $field) {
            add_settings_field(
                $field['name'],
                __( $field['title'], 'j_easy_booking_extended' ) ,
                function() use ($field) {
                    wceb_settings_input( array(
                        'type'              => 'text',
                        'id'                => $field['name'],
                        'name'              => $field['name'],
                        'value'             => '',
                        'description'       => __($field['title'], 'j_easy_booking_extended'),
                        'custom_attributes' => array(),
                        'class'             => ''
                    ));
                },
                'jebe_rates_settings',
                'jebe_rates_main_settings'
            );
        }

        register_setting(
            'jebe_extra_groups_settings',
            'jebe_extra_groups', 
            array( $this, 'sanitize_extra_groups_collection' )
        );

        add_settings_section(
            'jebe_extra_groups_main_settings',
            __( 'Extra groups', 'j_easy_booking_extended' ),
            array( $this, 'jebe_section_extra_group' ),
            'jebe_extra_groups_settings'
        );

        foreach ($this->extra_groups_fields as $key => $field) {
            add_settings_field(
                $field['name'],
                __( $field['title'], 'j_easy_booking_extended' ) ,
                function() use ($field) {
                    wceb_settings_input( array(
                        'type'              => 'text',
                        'id'                => $field['name'],
                        'name'              => $field['name'],
                        'value'             => '',
                        'description'       => __($field['title'], 'j_easy_booking_extended'),
                        'custom_attributes' => array(),
                        'class'             => ''
                    ));
                },
                'jebe_extra_groups_settings',
                'jebe_extra_groups_main_settings'
            );
        }

		
		wp_enqueue_style('admin_css_bootstrap', plugins_url('/myplugin/css/bootstrap.min.css'), false, '1.0.0', 'all');

	}

	/**
    *
    * Add settings link
    *
    **/
    public function jebe_add_admin_menu( ) {
        add_submenu_page( 'easy-booking', 'Hour bookings', 'Hour bookings', 'manage_options', 'jebe_settings', array( $this, 'jebe_admin_page' ) );
    }

    public function jebe_admin_page( ) {
    	if ( !current_user_can( 'manage_options' ) )  {
			wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
		}

		include('views/settings.php');
    }

    public function jebe_general_settings( ) {
    	settings_fields( 'jebe_general_settings' );
		do_settings_sections( 'jebe_general_settings' );
    }

    public function jebe_holidays_settings( ) {
    	settings_fields( 'jebe_holidays_settings' );
		do_settings_sections( 'jebe_holidays_settings' );
    }

    public function jebe_rates_settings( ) {
        settings_fields( 'jebe_rates_settings' );
        do_settings_sections( 'jebe_rates_settings' );
    }

    public function jebe_extra_groups_settings( ) {
        settings_fields( 'jebe_extra_groups_settings' );
        do_settings_sections( 'jebe_extra_groups_settings' );
    }

    public function jebe_holidays_dates_display( ) {
    	sort($this->holidays);
    	$holidays = $this->holidays;
    	include('views/holidays_list_view.php');
    }

    public function jebe_rates_display( ) {
        sort($this->rates);
        $rates = $this->rates;
        include('views/rates_list_view.php');
    }

    public function jebe_extra_groups_display( ) {
        sort($this->extra_groups);
        $extra_groups = $this->extra_groups;
        include('views/extra_groups_list_view.php');
    }

    public function jebe_section_general( ) {
    	echo '';
    }

    public function jebe_section_holiday( ) {
    	echo '';
    }

    public function jebe_section_rate( ) {
        echo '';
    }

    public function jebe_section_extra_group( ) {
        echo '';
    }

    public function jebe_field_default_hour_price( ) {
    	wceb_settings_input( array(
			'type'              => 'number',
			'id'                => 'jebe_default_hour_price',
			'name'              => 'jebe_settings[jebe_default_hour_price]',
			'value'             => isset( $this->options['jebe_default_hour_price'] ) ? absint( $this->options['jebe_default_hour_price'] ) : '',
			'description'       => __('Default price per hour.', 'j_easy_booking_extended'),
			'custom_attributes' => array(
				'min' 	=> 0,
				'step' 	=> '.01'
			)
		));
    }

    public function jebe_field_default_day_price( ) {
    	wceb_settings_input( array(
			'type'              => 'number',
			'id'                => 'jebe_default_day_price',
			'name'              => 'jebe_settings[jebe_default_day_price]',
			'value'             => isset( $this->options['jebe_default_day_price'] ) ? absint( $this->options['jebe_default_day_price'] ) : '',
			'description'       => __('Default price per day.', 'j_easy_booking_extended'),
			'custom_attributes' => array(
				'min' 	=> 0,
				'step' 	=> '.01'
			)
		));
    }

    public function jebe_field_default_weekend_price( ) {
    	wceb_settings_input( array(
			'type'              => 'number',
			'id'                => 'jebe_default_weekend_price',
			'name'              => 'jebe_settings[jebe_default_weekend_price]',
			'value'             => isset( $this->options['jebe_default_weekend_price'] ) ? absint( $this->options['jebe_default_weekend_price'] ) : '',
			'description'       => __('Default price per weekend.', 'j_easy_booking_extended'),
			'custom_attributes' => array(
				'min' 	=> 0,
				'step' 	=> '.01'
			)
		));
    }

    public function jebe_field_default_holiday_price( ) {
    	wceb_settings_input( array(
			'type'              => 'number',
			'id'                => 'jebe_default_holiday_price',
			'name'              => 'jebe_settings[jebe_default_holiday_price]',
			'value'             => isset( $this->options['jebe_default_holiday_price'] ) ? absint( $this->options['jebe_default_holiday_price'] ) : '',
			'description'       => __('Default price per holiday.', 'j_easy_booking_extended'),
			'custom_attributes' => array(
				'min' 	=> 0,
				'step' 	=> '.01'
			)
		));
    }

    public function jebe_field_allow_book_in_holiday( ) {
    	wceb_settings_checkbox( array(
			'id'          => 'jebe_allow_book_in_holiday',
			'name'        => 'jebe_settings[jebe_allow_book_in_holiday]',
			'description' => __( 'Check to allow bookings in holidays.', 'j_easy_booking_extended' ),
			'value'       => isset( $this->options['jebe_allow_book_in_holiday'] ) ? $this->options['jebe_allow_book_in_holiday'] : '',
			'cbvalue'     => 'on'
		));
    }

    public function jebe_field_min_bookable_hour( ) {
    	$options = array();
    	foreach (range(0, 23) as $key => $value) {
    		$options[] = sprintf("%02d", $value).':00';
    	}

    	wceb_settings_select( array(
			'id'          => 'jebe_min_bookable_hour',
			'name'        => 'jebe_settings[jebe_min_bookable_hour]',
			'description' => __( 'Min bookable hour.', 'j_easy_booking_extended' ),
			'value'       => isset( $this->options['jebe_min_bookable_hour'] ) ? $this->options['jebe_min_bookable_hour'] : '',
			'options'	  => $options
		));
    }

    public function jebe_field_max_bookable_hour( ) {
    	$options = array();
    	foreach (range(0, 23) as $key => $value) {
    		$options[] = sprintf("%02d", $value).':00';
    	}

    	wceb_settings_select( array(
			'id'          => 'jebe_max_bookable_hour',
			'name'        => 'jebe_settings[jebe_max_bookable_hour]',
			'description' => __( 'Min bookable hour.', 'j_easy_booking_extended' ),
			'value'       => isset( $this->options['jebe_max_bookable_hour'] ) ? $this->options['jebe_max_bookable_hour'] : '',
			'options'	  => $options
		));
    }

    public function jebe_field_stock_management( ) {
    	$options = array();
    	$options[0] = '';
    	$attributes =  wc_get_attribute_taxonomies();
    	if($attributes)
	        foreach ( $attributes as $attribute ) 
	            $options[$attribute->attribute_name] = $attribute->attribute_label;

    	wceb_settings_select( array(
			'id'          => 'jebe_stock_attribute',
			'name'        => 'jebe_settings[jebe_stock_attribute]',
			'description' => __( 'Stock will be calculated based only in this attribute. Two variations with this same attribute value will share stock', 'j_easy_booking_extended' ),
			'value'       => isset( $this->options['jebe_stock_attribute'] ) ? $this->options['jebe_stock_attribute'] : '',
			'options'	  => $options
		));
    }

    public function jebe_field_holiday( ) {
    	wceb_settings_input( array(
			'type'              => 'date',
			'id'                => 'jebe_holiday',
			'name'              => 'jebe_holiday',
			'value'             => '',
			'description'       => __('Add holiday date.', 'j_easy_booking_extended'),
			'custom_attributes' => array(),
			'class'				=> 'datepicker'
		));
    }

    public function jebe_field_rate_name( ) {
        wceb_settings_input( array(
            'type'              => 'text',
            'id'                => 'jebe_rate_name',
            'name'              => 'jebe_rate_name',
            'value'             => '',
            'description'       => __('Add rate name.', 'j_easy_booking_extended'),
            'custom_attributes' => array(),
            'class'             => ''
        ));
    }

    public function sanitize_values( $settings ) {

    	return $settings;
    }

    public function sanitize_collection( $settings ) {
    	
    	if( isset( $_POST ) ) {


	        if( array_key_exists( 'jebe_holiday', $_POST ) ) {
	        	
	    		if( isset( $_POST['delete'] ) ) {

	    			if( in_array( $_POST['delete'], $this->holidays ) )
			    		$this->holidays = array_diff( $this->holidays, array( $_POST['delete'] ) );

	    		}
                else {
                    
                    if( ! in_array( $_POST['jebe_holiday'], $this->holidays ) && ! ($_POST['jebe_holiday'] == '') )
                    
                        $this->holidays[] = $_POST['jebe_holiday'];

                }

                $settings = $this->holidays;
            }

            if( array_key_exists( 'jebe_rate_name', $_POST ) ) {
                
                $this->rates = array_filter( $this->rates, function( $rate ) {
                    return $rate['name'] !== $_POST['jebe_rate_name'];
                });
                
                if ( ( ! ($_POST['jebe_rate_name'] == '' ) && ( ! array_key_exists( 'delete', $_POST ) ) ) ){
    
                    $rate_values = array();

                    foreach ($this->rate_fields as $key => $field) {
                        $rate_values[$field['name']] = $_POST[$field['name']];
                    }

                    $this->rates[] = array( 
                        'name' => $_POST['jebe_rate_name'], 
                        'rates' => $rate_values );

                }
                $settings = $this->rates;
            }
        }

    	return $settings;
    }

    public function sanitize_extra_groups_collection( $settings ) {
        
        if( isset( $_POST ) ) {

            if( array_key_exists( 'extra_group_name', $_POST ) ) {

                $this->extra_groups = array_filter( $this->extra_groups, function( $extra_group ) {
                    return $extra_group['extra_group_name'] !== $_POST['extra_group_name'];
                });

                if ( ( ! ($_POST['extra_group_name'] == '' ) && ( ! array_key_exists( 'delete', $_POST ) ) ) ){

                    $extra_group = [
                        'extra_group_name' => $_POST['extra_group_name'],
                        'extras' => array_values(
                            array_filter($_POST['extra'], function($extra){
                                return $extra['extra_name'] !== '';
                            } )
                        )
                    ];

                    $this->extra_groups[] = $extra_group;
                }
            }

            $settings = $this->extra_groups;
        }

        return $settings;
    }
}

return new JEBE_Settings();
